@@ -186,9 +186,7 @@ common_user_list_get_instance (void)
 void
 common_user_list_cleanup (void)
 {
-    if (singleton)
-        g_object_unref (singleton);
-    singleton = NULL;
+    g_clear_object (&singleton);
 }
 
 static CommonUser *
@@ -1059,8 +1057,7 @@ common_user_list_finalize (GObject *object)
     if (priv->session_removed_signal)
         g_dbus_connection_signal_unsubscribe (priv->bus, priv->session_removed_signal);
     g_object_unref (priv->bus);
-    if (priv->passwd_monitor)
-        g_object_unref (priv->passwd_monitor);
+    g_clear_object (&priv->passwd_monitor);
 
     G_OBJECT_CLASS (common_user_list_parent_class)->finalize (object);
 }
