@@ -802,6 +802,7 @@ lightdm_greeter_connect_to_daemon (LightDMGreeter *greeter, GCancellable *cancel
 
 /**
  * lightdm_greeter_connect_to_daemon_finish:
+ * @greeter: The greeter the the request was done with
  * @result: A #GAsyncResult.
  * @error: return location for a #GError, or %NULL
  *
@@ -1952,8 +1953,7 @@ request_finalize (GObject *object)
     Request *request = REQUEST (object);
 
     g_free (request->dir);
-    if (request->cancellable)
-        g_object_unref (request->cancellable);
+    g_clear_object (&request->cancellable);
 
     G_OBJECT_CLASS (request_parent_class)->finalize (object);
 }
