@@ -76,6 +76,8 @@ struct GreeterPrivate
     gboolean guest_account_authenticated;
 
     /* Communication channels to communicate with */
+    int to_greeter_input;
+    int from_greeter_output;
     GIOChannel *to_greeter_channel;
     GIOChannel *from_greeter_channel;
     guint from_greeter_watch;
@@ -410,8 +412,7 @@ reset_session (Greeter *greeter)
     {
         g_signal_handlers_disconnect_matched (greeter->priv->authentication_session, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, greeter);
         session_stop (greeter->priv->authentication_session);
-        g_object_unref (greeter->priv->authentication_session);
-        greeter->priv->authentication_session = NULL;
+        g_clear_object (&greeter->priv->authentication_session);
     }
 
     greeter->priv->guest_account_authenticated = FALSE;
@@ -436,7 +437,7 @@ handle_login (Greeter *greeter, guint32 sequence_number, const gchar *username)
     if (greeter->priv->active_username)
         g_free (greeter->priv->active_username);
     greeter->priv->active_username = g_strdup (username);
-    g_object_notify (G_OBJECT (greeter), "active-username");
+    g_object_notify (G_OBJECT (greeter), GREETER_PROPERTY_ACTIVE_USERNAME);
 
     greeter->priv->authentication_sequence_number = sequence_number;
     g_signal_emit (greeter, signals[CREATE_SESSION], 0, &greeter->priv->authentication_session);
@@ -446,8 +447,8 @@ handle_login (Greeter *greeter, guint32 sequence_number, const gchar *username)
         return;
     }
 
-    g_signal_connect (G_OBJECT (greeter->priv->authentication_session), "got-messages", G_CALLBACK (pam_messages_cb), greeter);
-    g_signal_connect (G_OBJECT (greeter->priv->authentication_session), "authentication-complete", G_CALLBACK (authentication_complete_cb), greeter);
+    g_signal_connect (G_OBJECT (greeter->priv->authentication_session), SESSION_SIGNAL_GOT_MESSAGES, G_CALLBACK (pam_messages_cb), greeter);
+    g_signal_connect (G_OBJECT (greeter->priv->authentication_session), SESSION_SIGNAL_AUTHENTICATION_COMPLETE, G_CALLBACK (authentication_complete_cb), greeter);
 
     /* Use non-interactive service for autologin user */
     autologin_username = g_hash_table_lookup (greeter->priv->hints, "autologin-user");
@@ -550,8 +551,8 @@ handle_login_remote (Greeter *greeter, const gchar *session_name, const gchar *u
     g_signal_emit (greeter, signals[CREATE_SESSION], 0, &greeter->priv->authentication_session);
     if (greeter->priv->authentication_session)
     {
-        g_signal_connect (G_OBJECT (greeter->priv->authentication_session), "got-messages", G_CALLBACK (pam_messages_cb), greeter);
-        g_signal_connect (G_OBJECT (greeter->priv->authentication_session), "authentication-complete", G_CALLBACK (authentication_complete_cb), greeter);
+        g_signal_connect (G_OBJECT (greeter->priv->authentication_session), SESSION_SIGNAL_GOT_MESSAGES, G_CALLBACK (pam_messages_cb), greeter);
+        g_signal_connect (G_OBJECT (greeter->priv->authentication_session), SESSION_SIGNAL_AUTHENTICATION_COMPLETE, G_CALLBACK (authentication_complete_cb), greeter);
 
         /* Run the session process */
         session_set_pam_service (greeter->priv->authentication_session, service);
@@ -954,6 +955,7 @@ greeter_start (Session *session)
 {
     Greeter *greeter = GREETER (session);
     int to_greeter_pipe[2], from_greeter_pipe[2];
+    int to_greeter_output, from_greeter_input;
     gboolean result = FALSE;
     gchar *value;
     GError *error = NULL;
@@ -964,12 +966,16 @@ greeter_start (Session *session)
         g_warning ("Failed to create pipes: %s", strerror (errno));
         return FALSE;
     }
-    greeter->priv->to_greeter_channel = g_io_channel_unix_new (to_greeter_pipe[1]);
+    to_greeter_output = to_greeter_pipe[0];
+    greeter->priv->to_greeter_input = to_greeter_pipe[1];
+    greeter->priv->to_greeter_channel = g_io_channel_unix_new (greeter->priv->to_greeter_input);
     g_io_channel_set_encoding (greeter->priv->to_greeter_channel, NULL, &error);
     if (error)
         g_warning ("Failed to set encoding on to greeter channel to binary: %s\n", error->message);
     g_clear_error (&error);
-    greeter->priv->from_greeter_channel = g_io_channel_unix_new (from_greeter_pipe[0]);
+    greeter->priv->from_greeter_output = from_greeter_pipe[0];
+    from_greeter_input = from_greeter_pipe[1];
+    greeter->priv->from_greeter_channel = g_io_channel_unix_new (greeter->priv->from_greeter_output);
     g_io_channel_set_encoding (greeter->priv->from_greeter_channel, NULL, &error);
     if (error)
         g_warning ("Failed to set encoding on from greeter channel to binary: %s\n", error->message);
@@ -978,22 +984,22 @@ greeter_start (Session *session)
     greeter->priv->from_greeter_watch = g_io_add_watch (greeter->priv->from_greeter_channel, G_IO_IN | G_IO_HUP, read_cb, greeter);
 
     /* Let the greeter session know how to communicate with the daemon */
-    value = g_strdup_printf ("%d", from_greeter_pipe[1]);
+    value = g_strdup_printf ("%d", from_greeter_input);
     session_set_env (SESSION (greeter), "LIGHTDM_TO_SERVER_FD", value);
     g_free (value);
-    value = g_strdup_printf ("%d", to_greeter_pipe[0]);
+    value = g_strdup_printf ("%d", to_greeter_output);
     session_set_env (SESSION (greeter), "LIGHTDM_FROM_SERVER_FD", value);
     g_free (value);
 
     /* Don't allow the daemon end of the pipes to be accessed in child processes */
-    fcntl (to_greeter_pipe[1], F_SETFD, FD_CLOEXEC);
-    fcntl (from_greeter_pipe[0], F_SETFD, FD_CLOEXEC);
+    fcntl (greeter->priv->to_greeter_input, F_SETFD, FD_CLOEXEC);
+    fcntl (greeter->priv->from_greeter_output, F_SETFD, FD_CLOEXEC);
 
     result = SESSION_CLASS (greeter_parent_class)->start (session);
 
     /* Close the session ends of the pipe */
-    close (to_greeter_pipe[0]);
-    close (from_greeter_pipe[1]);
+    close (to_greeter_output);
+    close (from_greeter_input);
 
     return result;
 }
@@ -1029,14 +1035,14 @@ greeter_init (Greeter *greeter)
     greeter->priv->read_buffer = secure_malloc (greeter, HEADER_SIZE);
     greeter->priv->hints = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
     greeter->priv->use_secure_memory = config_get_boolean (config_get_instance (), "LightDM", "lock-memory");
+    greeter->priv->to_greeter_input = -1;
+    greeter->priv->from_greeter_output = -1;
 }
 
 static void
 greeter_finalize (GObject *object)
 {
-    Greeter *self;
-
-    self = GREETER (object);
+    Greeter *self = GREETER (object);
 
     g_free (self->priv->pam_service);
     g_free (self->priv->autologin_pam_service);
@@ -1049,6 +1055,8 @@ greeter_finalize (GObject *object)
         g_signal_handlers_disconnect_matched (self->priv->authentication_session, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, self);
         g_object_unref (self->priv->authentication_session);
     }
+    close (self->priv->to_greeter_input);
+    close (self->priv->from_greeter_output);
     if (self->priv->to_greeter_channel)
         g_io_channel_unref (self->priv->to_greeter_channel);
     if (self->priv->from_greeter_channel)
@@ -1101,7 +1109,7 @@ greeter_class_init (GreeterClass *klass)
     object_class->set_property = greeter_set_property;
 
     signals[CONNECTED] =
-        g_signal_new ("connected",
+        g_signal_new (GREETER_SIGNAL_CONNECTED,
                       G_TYPE_FROM_CLASS (klass),
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (GreeterClass, connected),
@@ -1110,7 +1118,7 @@ greeter_class_init (GreeterClass *klass)
                       G_TYPE_NONE, 0);
 
     signals[CREATE_SESSION] =
-        g_signal_new ("create-session",
+        g_signal_new (GREETER_SIGNAL_CREATE_SESSION,
                       G_TYPE_FROM_CLASS (klass),
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (GreeterClass, create_session),
@@ -1120,7 +1128,7 @@ greeter_class_init (GreeterClass *klass)
                       SESSION_TYPE, 0);
 
     signals[START_SESSION] =
-        g_signal_new ("start-session",
+        g_signal_new (GREETER_SIGNAL_START_SESSION,
                       G_TYPE_FROM_CLASS (klass),
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (GreeterClass, start_session),
@@ -1131,8 +1139,8 @@ greeter_class_init (GreeterClass *klass)
 
     g_object_class_install_property (object_class,
                                      PROP_ACTIVE_USERNAME,
-                                     g_param_spec_string ("active-username",
-                                                          "active-username",
+                                     g_param_spec_string (GREETER_PROPERTY_ACTIVE_USERNAME,
+                                                          GREETER_PROPERTY_ACTIVE_USERNAME,
                                                           "Active username",
                                                           NULL,
                                                           G_PARAM_READABLE));
