@@ -65,6 +65,9 @@ struct SeatPrivate
 
     /* TRUE if stopped */
     gboolean stopped;
+    
+    /* The greeter to be started to replace the current one */
+    Greeter *replacement_greeter;
 };
 
 static void seat_logger_iface_init (LoggerInterface *iface);
@@ -256,14 +259,11 @@ seat_set_active_session (Seat *seat, Session *session)
     }
 
     /* Lock previous sessions */
-    if (seat->priv->active_session)
-    {
-        if (session != seat->priv->active_session && !IS_GREETER (seat->priv->active_session))
-            session_lock (seat->priv->active_session);
-        g_object_unref (seat->priv->active_session);
-    }
+    if (seat->priv->active_session && session != seat->priv->active_session && !IS_GREETER (seat->priv->active_session))
+        session_lock (seat->priv->active_session);
 
     session_activate (session);
+    g_clear_object (&seat->priv->active_session);
     seat->priv->active_session = g_object_ref (session);
 }
 
@@ -309,8 +309,7 @@ void
 seat_set_externally_activated_session (Seat *seat, Session *session)
 {
     g_return_if_fail (seat != NULL);
-    if (seat->priv->active_session)
-        g_object_unref (seat->priv->active_session);
+    g_clear_object (&seat->priv->active_session);
     seat->priv->active_session = g_object_ref (session);
 }
 
@@ -577,8 +576,7 @@ switch_to_greeter_from_failed_session (Seat *seat, Session *session)
     }
     else
     {
-        if (seat->priv->session_to_activate)
-            g_object_unref (seat->priv->session_to_activate);
+        g_clear_object (&seat->priv->session_to_activate);
         seat->priv->session_to_activate = g_object_ref (greeter_session);
 
         if (can_share_display_server (seat, session_get_display_server (session)))
@@ -665,8 +663,7 @@ run_session (Seat *seat, Session *session)
     if (session == seat->priv->session_to_activate)
     {
         seat_set_active_session (seat, session);
-        g_object_unref (seat->priv->session_to_activate);
-        seat->priv->session_to_activate = NULL;
+        g_clear_object (&seat->priv->session_to_activate);
     }
     else if (seat->priv->active_session)
     {
@@ -710,8 +707,7 @@ greeter_active_username_changed_cb (Greeter *greeter, GParamSpec *pspec, Seat *s
 
     session = find_user_session (seat, greeter_get_active_username (greeter), seat->priv->active_session);
 
-    if (seat->priv->next_session)
-        g_object_unref (seat->priv->next_session);
+    g_clear_object (&seat->priv->next_session);
     seat->priv->next_session = session ? g_object_ref (session) : NULL;
 
     SEAT_GET_CLASS (seat)->set_next_session (seat, session);
@@ -759,20 +755,11 @@ session_stopped_cb (Session *session, Seat *seat)
     g_signal_handlers_disconnect_matched (session, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, seat);
     seat->priv->sessions = g_list_remove (seat->priv->sessions, session);
     if (session == seat->priv->active_session)
-    {
-        g_object_unref (seat->priv->active_session);
-        seat->priv->active_session = NULL;
-    }
+        g_clear_object (&seat->priv->active_session);
     if (session == seat->priv->next_session)
-    {
-        g_object_unref (seat->priv->next_session);
-        seat->priv->next_session = NULL;
-    }
+        g_clear_object (&seat->priv->next_session);
     if (session == seat->priv->session_to_activate)
-    {
-        g_object_unref (seat->priv->session_to_activate);
-        seat->priv->session_to_activate = NULL;
-    }
+        g_clear_object (&seat->priv->session_to_activate);
 
     display_server = session_get_display_server (session);
     if (!display_server)
@@ -793,10 +780,7 @@ session_stopped_cb (Session *session, Seat *seat)
     /* We were waiting for this session, but it didn't start :( */
     // FIXME: Start a greeter on this?
     if (session == seat->priv->session_to_activate)
-    {
-        g_object_unref (seat->priv->session_to_activate);
-        seat->priv->session_to_activate = NULL;
-    }
+        g_clear_object (&seat->priv->session_to_activate);
 
     if (seat->priv->stopping)
     {
@@ -804,9 +788,28 @@ session_stopped_cb (Session *session, Seat *seat)
         g_object_unref (session);
         return;
     }
+    
+    /* If there is a pending replacement greeter, start it */
+    if (IS_GREETER (session) && seat->priv->replacement_greeter)
+    {
+        Greeter *replacement_greeter = seat->priv->replacement_greeter;
+        seat->priv->replacement_greeter = NULL;
+        
+        if (session_get_is_authenticated (SESSION (replacement_greeter)))
+        {
+            l_debug (seat, "Greeter stopped, running session");
+            run_session (seat, SESSION (replacement_greeter));
+        }
+        else
+        {
+            l_debug (seat, "Greeter stopped, starting session authentication");
+            start_session (seat, SESSION (replacement_greeter));
+        }
 
+        g_object_unref (replacement_greeter);
+    }
     /* If this is the greeter session then re-use this display server */
-    if (IS_GREETER (session) &&
+    else if (IS_GREETER (session) &&
         can_share_display_server (seat, display_server) &&
         greeter_get_start_session (GREETER (session)))
     {
@@ -1214,8 +1217,7 @@ greeter_start_session_cb (Greeter *greeter, SessionType type, const gchar *sessi
     }
 
     /* Switch to this session when it is ready */
-    if (seat->priv->session_to_activate)
-        g_object_unref (seat->priv->session_to_activate);
+    g_clear_object (&seat->priv->session_to_activate);
     seat->priv->session_to_activate = g_object_ref (session);
 
     /* Return to existing session if it is open */
@@ -1446,8 +1448,7 @@ seat_switch_to_greeter (Seat *seat)
     if (!greeter_session)
         return FALSE;
 
-    if (seat->priv->session_to_activate)
-        g_object_unref (seat->priv->session_to_activate);
+    g_clear_object (&seat->priv->session_to_activate);
     seat->priv->session_to_activate = g_object_ref (greeter_session);
 
     display_server = create_display_server (seat, SESSION (greeter_session));
@@ -1479,8 +1480,7 @@ switch_authentication_complete_cb (Session *session, Seat *seat)
         else
         {
             l_debug (seat, "Session authenticated, starting display server");
-            if (seat->priv->session_to_activate)
-                g_object_unref (seat->priv->session_to_activate);
+            g_clear_object (&seat->priv->session_to_activate);
             seat->priv->session_to_activate = g_object_ref (session);
             display_server = create_display_server (seat, session);
             session_set_display_server (session, display_server);
@@ -1518,8 +1518,7 @@ switch_authentication_complete_cb (Session *session, Seat *seat)
     }
     else
     {
-        if (seat->priv->session_to_activate)
-            g_object_unref (seat->priv->session_to_activate);
+        g_clear_object (&seat->priv->session_to_activate);
         seat->priv->session_to_activate = g_object_ref (greeter_session);
 
         display_server = create_display_server (seat, SESSION (greeter_session));
@@ -1595,8 +1594,7 @@ seat_switch_to_guest (Seat *seat, const gchar *session_name)
 
     display_server = create_display_server (seat, session);
 
-    if (seat->priv->session_to_activate)
-        g_object_unref (seat->priv->session_to_activate);
+    g_clear_object (&seat->priv->session_to_activate);
     seat->priv->session_to_activate = g_object_ref (session);
     session_set_pam_service (session, seat_get_string_property (seat, "pam-autologin-service"));
     session_set_display_server (session, display_server);
@@ -1608,14 +1606,17 @@ gboolean
 seat_lock (Seat *seat, const gchar *username)
 {
     Greeter *greeter_session;
-    DisplayServer *display_server;
-    gboolean existing = FALSE;
+    DisplayServer *display_server = NULL;
+    gboolean reset_existing = FALSE;
+    gboolean reuse_xserver = FALSE;
 
     g_return_val_if_fail (seat != NULL, FALSE);
 
     if (!seat_get_can_switch (seat))
         return FALSE;
 
+    // FIXME: If already locked then don't bother...
+
     l_debug (seat, "Locking");
 
     /* Switch to greeter we can reuse */
@@ -1624,10 +1625,23 @@ seat_lock (Seat *seat, const gchar *username)
     {
         l_debug (seat, "Switching to existing greeter");
         set_greeter_hints (seat, greeter_session);
-        existing = TRUE;
+        reset_existing = TRUE;
     }
     else
     {
+        /* If the existing greeter can't be reused, stop it and reuse its display server */
+        greeter_session = find_greeter_session (seat);
+        if (greeter_session)
+        {
+            display_server = session_get_display_server (SESSION (greeter_session));
+            if (!session_get_is_stopping (SESSION (greeter_session)))
+            {
+                l_debug (seat, "Stopping session");
+                session_stop (SESSION (greeter_session));
+            }
+            reuse_xserver = TRUE;
+        }
+        
         greeter_session = create_greeter_session (seat);
         if (!greeter_session)
             return FALSE;
@@ -1637,7 +1651,7 @@ seat_lock (Seat *seat, const gchar *username)
     if (username)
         greeter_set_hint (greeter_session, "select-user", username);
 
-    if (existing)
+    if (reset_existing)
     {
         greeter_reset (greeter_session);
         seat_set_active_session (seat, SESSION (greeter_session));
@@ -1645,14 +1659,21 @@ seat_lock (Seat *seat, const gchar *username)
     }
     else
     {
-        display_server = create_display_server (seat, SESSION (greeter_session));
-
-        if (seat->priv->session_to_activate)
-            g_object_unref (seat->priv->session_to_activate);
-        seat->priv->session_to_activate = g_object_ref (greeter_session);
+        if (!reuse_xserver)
+            display_server = create_display_server (seat, SESSION (greeter_session));
         session_set_display_server (SESSION (greeter_session), display_server);
 
-        return display_server_start (display_server);
+        g_clear_object (&seat->priv->session_to_activate);
+        seat->priv->session_to_activate = g_object_ref (greeter_session);
+
+        if (reuse_xserver)
+        {
+            g_clear_object (&seat->priv->replacement_greeter);
+            seat->priv->replacement_greeter = g_object_ref (greeter_session);
+            return TRUE;
+        }
+        else
+            return display_server_start (display_server);
     }
 }
 
@@ -1720,8 +1741,7 @@ seat_real_start (Seat *seat)
         {
             DisplayServer *display_server;
 
-            if (seat->priv->session_to_activate)
-                g_object_unref (seat->priv->session_to_activate);
+            g_clear_object (&seat->priv->session_to_activate);
             seat->priv->session_to_activate = g_object_ref (session);
 
             display_server = create_display_server (seat, session);
@@ -1750,8 +1770,7 @@ seat_real_start (Seat *seat)
             return FALSE;
         }
 
-        if (seat->priv->session_to_activate)
-            g_object_unref (seat->priv->session_to_activate);
+        g_clear_object (&seat->priv->session_to_activate);
         seat->priv->session_to_activate = g_object_ref (greeter_session);
         session = SESSION (greeter_session);
 
@@ -1866,11 +1885,9 @@ seat_init (Seat *seat)
 static void
 seat_finalize (GObject *object)
 {
-    Seat *self;
+    Seat *self = SEAT (object);
     GList *link;
 
-    self = SEAT (object);
-
     g_free (self->priv->name);
     g_hash_table_unref (self->priv->properties);
     for (link = self->priv->display_servers; link; link = link->next)
@@ -1885,12 +1902,10 @@ seat_finalize (GObject *object)
         g_signal_handlers_disconnect_matched (session, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, self);
     }
     g_list_free_full (self->priv->sessions, g_object_unref);
-    if (self->priv->active_session)
-        g_object_unref (self->priv->active_session);
-    if (self->priv->next_session)
-        g_object_unref (self->priv->next_session);
-    if (self->priv->session_to_activate)
-        g_object_unref (self->priv->session_to_activate);
+    g_clear_object (&self->priv->active_session);
+    g_clear_object (&self->priv->next_session);
+    g_clear_object (&self->priv->session_to_activate);
+    g_clear_object (&self->priv->replacement_greeter);
 
     G_OBJECT_CLASS (seat_parent_class)->finalize (object);
 }
