@@ -136,8 +136,7 @@ session_set_config (Session *session, SessionConfig *config)
 {
     g_return_if_fail (session != NULL);
 
-    if (session->priv->config)
-        g_object_unref (session->priv->config);
+    g_clear_object (&session->priv->config);
     session->priv->config = g_object_ref (config);
 }
 
@@ -248,11 +247,7 @@ void
 session_set_x_authority (Session *session, XAuthority *authority, gboolean use_system_location)
 {
     g_return_if_fail (session != NULL);
-    if (session->priv->x_authority)
-    {
-        g_object_unref (session->priv->x_authority);
-        session->priv->x_authority = NULL;
-    }
+    g_clear_object (&session->priv->x_authority);
     if (authority)
         session->priv->x_authority = g_object_ref (authority);
     session->priv->x_authority_use_system_location = use_system_location;
@@ -486,9 +481,7 @@ from_child_cb (GIOChannel *source, GIOCondition condition, gpointer data)
     {
         g_free (session->priv->username);
         session->priv->username = username;
-        if (session->priv->user)
-            g_object_unref (session->priv->user);
-        session->priv->user = NULL;
+        g_clear_object (&session->priv->user);
     }
     else
         g_free (username);
@@ -926,10 +919,8 @@ session_finalize (GObject *object)
     Session *self = SESSION (object);
     int i;
 
-    if (self->priv->config)
-        g_object_unref (self->priv->config);
-    if (self->priv->display_server)
-        g_object_unref (self->priv->display_server);
+    g_clear_object (&self->priv->config);
+    g_clear_object (&self->priv->display_server);
     if (self->priv->pid)
         kill (self->priv->pid, SIGKILL);
     close (self->priv->to_child_input);
@@ -941,8 +932,7 @@ session_finalize (GObject *object)
     if (self->priv->child_watch)
         g_source_remove (self->priv->child_watch);
     g_free (self->priv->username);
-    if (self->priv->user)
-        g_object_unref (self->priv->user);
+    g_clear_object (&self->priv->user);
     g_free (self->priv->pam_service);
     for (i = 0; i < self->priv->messages_length; i++)
         g_free ((char *) self->priv->messages[i].msg);
@@ -951,8 +941,7 @@ session_finalize (GObject *object)
     g_free (self->priv->log_filename);
     g_free (self->priv->tty);
     g_free (self->priv->xdisplay);
-    if (self->priv->x_authority)
-        g_object_unref (self->priv->x_authority);
+    g_clear_object (&self->priv->x_authority);
     g_free (self->priv->remote_host_name);
     g_free (self->priv->login1_session_id);
     g_free (self->priv->console_kit_cookie);
