@@ -158,8 +158,7 @@ stopped_cb (Process *process, XServerXVNC *server)
 {
     l_debug (server, "Xvnc server stopped");
 
-    g_object_unref (server->priv->x_server_process);
-    server->priv->x_server_process = NULL;
+    g_clear_object (&server->priv->x_server_process);
 
     x_server_local_release_display_number (x_server_get_display_number (X_SERVER (server)));
 
@@ -183,7 +182,7 @@ x_server_xvnc_start (DisplayServer *display_server)
 {
     XServerXVNC *server = X_SERVER_XVNC (display_server);
     XAuthority *authority;
-    gboolean result;
+    gboolean result, backup_logs;
     gchar *filename, *run_dir, *dir, *log_file, *absolute_command;
     GString *command;
     gchar hostname[1024], *number;
@@ -195,14 +194,15 @@ x_server_xvnc_start (DisplayServer *display_server)
 
     server->priv->x_server_process = process_new (run_cb, server);
     process_set_clear_environment (server->priv->x_server_process, TRUE);
-    g_signal_connect (server->priv->x_server_process, "got-signal", G_CALLBACK (got_signal_cb), server);
-    g_signal_connect (server->priv->x_server_process, "stopped", G_CALLBACK (stopped_cb), server);
+    g_signal_connect (server->priv->x_server_process, PROCESS_SIGNAL_GOT_SIGNAL, G_CALLBACK (got_signal_cb), server);
+    g_signal_connect (server->priv->x_server_process, PROCESS_SIGNAL_STOPPED, G_CALLBACK (stopped_cb), server);
 
     /* Setup logging */
     filename = g_strdup_printf ("%s.log", display_server_get_name (display_server));
     dir = config_get_string (config_get_instance (), "LightDM", "log-directory");
     log_file = g_build_filename (dir, filename, NULL);
-    process_set_log_file (server->priv->x_server_process, log_file, FALSE);
+    backup_logs = config_get_boolean (config_get_instance (), "LightDM", "backup-logs");  
+    process_set_log_file (server->priv->x_server_process, log_file, FALSE, backup_logs ? LOG_MODE_BACKUP_AND_TRUNCATE : LOG_MODE_APPEND);
     l_debug (display_server, "Logging to %s", log_file);
     g_free (log_file);
     g_free (filename);
@@ -291,12 +291,9 @@ x_server_xvnc_init (XServerXVNC *server)
 static void
 x_server_xvnc_finalize (GObject *object)
 {
-    XServerXVNC *self;
+    XServerXVNC *self = X_SERVER_XVNC (object);
 
-    self = X_SERVER_XVNC (object);
-
-    if (self->priv->x_server_process)
-        g_object_unref (self->priv->x_server_process);
+    g_clear_object (&self->priv->x_server_process);
     g_free (self->priv->command);
     g_free (self->priv->authority_file);
 
