@@ -90,12 +90,9 @@ x_server_set_authority (XServer *server, XAuthority *authority)
 {
     g_return_if_fail (server != NULL);
 
-    if (server->priv->authority)
-        g_object_unref (server->priv->authority);
+    g_clear_object (&server->priv->authority);
     if (authority)
         server->priv->authority = g_object_ref (authority);
-    else
-        server->priv->authority = NULL;
 }
 
 XAuthority *
@@ -204,14 +201,11 @@ x_server_init (XServer *server)
 static void
 x_server_finalize (GObject *object)
 {
-    XServer *self;
-
-    self = X_SERVER (object);
+    XServer *self = X_SERVER (object);
 
     g_free (self->priv->hostname);
     g_free (self->priv->address);
-    if (self->priv->authority)
-        g_object_unref (self->priv->authority);
+    g_clear_object (&self->priv->authority);
     if (self->priv->connection)
         xcb_disconnect (self->priv->connection);
 
